﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

using Livet;
using Livet.Commands;
using Livet.Messaging.Windows;

using LivetSampleProject.Models;

namespace LivetSampleProject.ViewModels
{
    public class MemberViewModel : ViewModel,IDataErrorInfo
    {
        private Member _model;
        private Dictionary<string, string> _errors = new Dictionary<string, string>();

        public MemberViewModel(Member m,MainWindowViewModel parent)
        {
            _model = m;
            MainWindowViewModel = parent;

            //入力値やエラー情報の初期化
            InitializeInput();
            
            //ModelのPropertyChangedイベントはBindNotifyChangedを使用してハンドルします。
            //こうする事で弱イベントによるModelイベントの購読が行われます。
            //イベントハンドラのライフサイクルはこのViewModelと同じなのでリークしませんし、
            //イベントハンドラがViewModelより先に消えません。
            ViewModelHelper.BindNotifyChanged(
                _model,
                this,
                (sender, e) =>
                {
                    RaisePropertyChanged(e.PropertyName);
                    if (e.PropertyName == "Birthday")
                    {
                        RaisePropertyChanged("Age");
                    }
                });
        }

        #region Modelプロパティのラッパー

        public string Name
        {
            get
            { return _model.Name; }
            set
            { _model.Name = value; }
        }

        public DateTime Birthday
        {
            get { return _model.Birthday; }
            set { _model.Birthday = value; }
        }

        public string Memo
        {
            get
            { return _model.Memo; }
            set
            { _model.Memo = value; }
        }

        #endregion

        #region ViewModelオリジナルのプロパティ

        public int Age
        {
            get
            { return (DateTime.Now - Birthday).Days / 365; }
        }

        bool _Checked;
        //チェックされているかを取得、または設定します。
        public bool Checked
        {
            get
            { return _Checked; }
            set
            {
                if (_Checked == value)
                    return;
                _Checked = value;

                if (Checked)
                {
                    MainWindowViewModel.CheckedViewModel.Add(this);
                }
                else
                {
                    MainWindowViewModel.CheckedViewModel.Remove(this);
                }

                RaisePropertyChanged("Checked");
            }
        }

        public MainWindowViewModel MainWindowViewModel
        {
            get;
            private set;
        }

        #endregion

        #region 入力用プロパティ

        private string _inputName;

        //名前の入力用プロパティです。
        public string InputName
        {
            get { return _inputName; }
            set 
            {
                _inputName = value;

                if (string.IsNullOrEmpty(_inputName.Trim()))
                {
                    this["InputName"] = "名前は必須です";
                }
                else
                {
                    this["InputName"] = null;
                }
            }
        }

        private string _inputBirthday;

        //生年月日の入力用プロパティです。
        public string InputBirthday
        {
            get { return _inputBirthday; }
            set 
            {
                _inputBirthday = value;

                DateTime inputDateTime;

                if (string.IsNullOrEmpty(_inputBirthday))
                {
                    this["InputBirthday"] = "生年月日は必須です";
                }
                else if (!DateTime.TryParse(_inputBirthday,out inputDateTime))
                {
                    this["InputBirthday"] = "年月日として不正な形式です";
                }
                else if (inputDateTime > DateTime.Now)
                {
                    this["InputBirthday"] = "未来の日付は指定できません";
                }
                else
                {
                    this["InputBirthday"] = null;
                }
            }
        }
    
        private string _inputMemo;

        //備考の入力用プロパティです。
        public string InputMemo
        {
            get { return _inputMemo; }
            set { _inputMemo = value; }
        }

        #endregion

        //入力値用プロパティを初期化します
        private void InitializeInput()
        {
            _inputName = _model.Name;
            if (_model.Birthday != DateTime.MinValue)
            {
                _inputBirthday = _model.Birthday.ToString("yyyy/MM/dd");
            }
            _inputMemo = _model.Memo;

            _errors.Clear();
        }

        #region SaveCommand
        ViewModelCommand _SaveCommand;

        public ViewModelCommand SaveCommand
        {
            get
            {
                if (_SaveCommand == null)
                    _SaveCommand = new ViewModelCommand(Save, CanSave);
                return _SaveCommand;
            }
        }

        private bool CanSave()
        {
            if(!string.IsNullOrEmpty(Error))
            {
                return false;
            }

            if (string.IsNullOrEmpty(InputName) || string.IsNullOrEmpty(InputBirthday))
            {
                return false;
            }

            return true;
        }

        private void Save()
        {
            Name = InputName;
            Birthday = DateTime.Parse(InputBirthday);
            Memo = InputMemo;

            if (!_model.IsIncludedInMainCollection)
            {
                _model.AddThisToMainCollection();
            }

            //Viewに画面遷移用メッセージを送信しています。
            //Viewは対応するメッセージキーを持つInteractionTransitionMessageTriggerでこのメッセージを受信します。
            Messenger.Raise(new WindowActionMessage("Close", WindowAction.Close));
        }
        #endregion
        
        #region CancelCommand
        ViewModelCommand _CancelCommand;

        public ViewModelCommand CancelCommand
        {
            get
            {
                if (_CancelCommand == null)
                    _CancelCommand = new ViewModelCommand(Cancel);
                return _CancelCommand;
            }
        }

        private void Cancel()
        {
            //入力情報初期化
            InitializeInput();

            //Viewに画面遷移用メッセージを送信しています。
            //Viewは対応するメッセージキーを持つInteractionTransitionMessageTriggerでこのメッセージを受信します。
            Messenger.Raise(new WindowActionMessage("Close", WindowAction.Close));;
        }
        #endregion
           
        #region RemoveCommand
        ViewModelCommand _RemoveCommand;

        public ViewModelCommand RemoveCommand
        {
            get
            {
                if (_RemoveCommand == null)
                    _RemoveCommand = new ViewModelCommand(Remove, CanRemove);
                return _RemoveCommand;
            }
        }

        private bool CanRemove()
        {
            return Checked;
        }

        private void Remove()
        {
            _model.RemoveThisFromMainCollection();

            if (MainWindowViewModel.CheckedViewModel.Contains(this))
            {
                MainWindowViewModel.CheckedViewModel.Remove(this);
            }
        }
        #endregion

        #region IDataErrorInfo
        public string Error
        {
            get 
            {
                List<string> errorPropertyList = new List<string>();

                if (!string.IsNullOrEmpty(this["InputName"]))
                {
                    errorPropertyList.Add("名前");
                }

                if (!string.IsNullOrEmpty(this["InputBirthday"]))
                {
                    errorPropertyList.Add("生年月日");
                }

                if (errorPropertyList.Count != 0)
                {
                    return string.Join("・", errorPropertyList.ToArray()) + "が不正です";
                }

                return null;
            }
        }

        public string this[string columnName]
        {
            get 
            {
                if (_errors.Keys.Contains(columnName))
                {
                    return _errors[columnName];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                _errors[columnName] = value;
                RaisePropertyChanged("Error");
                SaveCommand.RaiseCanExecuteChanged();
            }
        }
        #endregion
    }
}
